# sage_setup: distribution = sagemath-environment
# Sage version information for Python scripts
# This file is auto-generated by the update-version script, do not edit!
version = '10.6.30'
date = '2025-10-05'
banner = 'passagemath version 10.6.30, Release Date: 2025-10-05'
